'''
switch_to path contextmanager (with)
http://ralsina.me/weblog/posts/BB963.html
'''
import os
import pathlib
from contextlib import contextmanager
from loguru import logger


@contextmanager
def switch_to(path=pathlib.Path().home()):
    '''
    switch to path

    with switch_to(path):
        pass  # do stuff
    '''
    old_dir = os.getcwd()

    try:
        path = pathlib.Path(path)
    except Exception as exc:
        logger.error(exc)
        raise

    if not path.is_dir():  # pragma: no cover
        msg = '*{}* is not a directory or does not exist.'
        raise Exception(msg.format(path))

    os.chdir(path)
    yield
    os.chdir(old_dir)



''' freemt __init__.py '''
# import sys
# from pathlib import Path
# import importlib

from .switch_to import switch_to
from .arun import arun
from .make_url import make_url
from .httpx_get import httpx_get
from .save_tempfile import save_tempfile
from .logger_level import logger_level

# sys.path.insert(0, '..')

# __file__ = '__init__.py'
_ = '''
for elm in Path(__file__).parent.glob('*.py'):
    stem = elm.stem
    _ = importlib.import_module(stem, package='freemt_utils')
    globals().update({stem: getattr(_, stem)})
# '''

# version__ = '0.0.1'
# date__ = '2020.2.12'
__version__ = '0.0.4'
__date__ = '2020.2.15'
VERSION = tuple(__version__.split('.'))



'''
set logger level of loguru DEBUG INFO
'''
# pylint: disable=broad-except

import sys
from loguru import logger


def logger_level(level: str = 'DEBUG') -> None:
    ''' set logger level'''

    try:
        logger.remove()  # noqa
        logger.add(sys.stderr, level=level.upper())  # noqa
    except Exception:
        print(sys.exc_info()[:2])



'''
https://stackoverflow.com/questions/55590343/asyncio-run-or-run-until-complete
'''

import sys
import asyncio


def arun(awt):
    ''' arun awaitable'''
    if sys.version_info >= (3, 7):
        return asyncio.run(awt)  # pylint: disable=no-member

    # Emulate asyncio.run() on older versions
    # loop = asyncio.new_event_loop()
    # asyncio.set_event_loop(loop)
    loop = asyncio.get_event_loop()

    try:
        return loop.run_until_complete(awt)
    except (KeyboardInterrupt, SystemExit):
        pass
    # finally:
        # loop.close()
        # asyncio.set_event_loop(None)



# coding: utf-8
"""
save to tempfile.NamedTemporaryFile
    with tempfile.NamedTemporaryFile('w', delete=False, suffix='.html') as f:
        # url = 'file://' + f.name
        f.write(html)
    webbrowser.open('file://' + f.name)

https://stackoverflow.com/questions/21437386/launch-html-code-in-browser-that-is-generated-by-beautifulsoup-straight-from-p

savetofile(text, filename='out.txt', encoding='utf-8', start=False)

Open file in firefox if set

based on savetofile
"""
import os
import pathlib
from pathlib import Path
import tempfile
import subprocess
import platform
import cchardet

# from nose.tools import (eq_, with_setup)

from loguru import logger as LOGGER


# def savetofile(text, filename='out.html', encoding='utf-8', start=False):
# def save_tempfile(text, filename=None, encoding='ascii', start=True):
def save_tempfile(text, filename=None, encoding='utf-8', start=True):  # pylint: disable=too-many-branches, too-many-return-statements
    """
    Save text (str) to file (default to tmp.txt) or filepath (Path(str))
    """
    if filename is None:
        filename = tempfile.NamedTemporaryFile('w', delete=False, suffix='.html').name

    if text is None:
        LOGGER.warning(f'Input is None, exiting...')
        return ''

    if isinstance(text, bytes):
        _ = cchardet.detect(text)
        enc = _['encoding']
        if enc is None:
            enc = 'UTF-8'

        text = text.decode(enc)

    if not isinstance(text, str):
        try:
            text = str(text)
        except Exception:
            LOGGER.warning(" Not a str, cant convert to a str, exiting")
            return ''

    if isinstance(filename, pathlib.Path):
        fpath = filename.absolute()
    else:
        try:
            fpath = Path(filename).absolute()
        except Exception as exc:
            LOGGER.error(" fpath = Path(filename).absolute() Exception: %s, exiting" % exc)
            return ''

    if encoding.lower() in ['utf8', 'utf-8']:
        encoding = 'utf-8'

    #  if encoding == 'ascii': ncr xmlcharrefreplace
    if encoding not in ['utf-8']:
        text = text.encode(encoding, errors='xmlcharrefreplace').decode(encoding)

    with open(str(fpath), 'w', encoding=encoding) as fhandle:
        fhandle.write(text)
        # LOGGER.info("\n\tSaved to %s (encoding: %s)", str(fpath), encoding)
        LOGGER.info("\n\tSaved to %s" % str(fpath))

    if start:
        # modi

        # 'start' only valid in Windows
        platform_ = platform.system()
        if not platform_ == 'Windows':
            LOGGER.info('Platform is %s, exiting...' % platform_)
            return fpath.__str__()

        try:
            os.startfile(str(fpath))
            return fpath.__str__()
        except Exception as exc:
            LOGGER.debug('startfile didnt work ...: %s' % exc)
            LOGGER.info("Trying firefox in C:...")

        proc = subprocess.Popen([r"C:\Program Files\Mozilla Firefox\firefox.exe", str(fpath)], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        LOGGER.info("Opening file in Firefox...")
        err, out = proc.communicate()
        if err:
            LOGGER.error("Start file in firefox errors: %s, out: %s", err, out)

        return fpath.__str__()

    return str(fpath)


_ = '''
def my_setup(level=logging.DEBUG):
    """my_setup."""

    fmt = '%(name)s-%(filename)s[ln:%(lineno)d]:'
    fmt += '%(levelname)s: %(message)s'
    logging.basicConfig(format=fmt, level=level)


@with_setup(my_setup)
def test_():
    """test_+++."""

    out = 1
    eq_(1, out)
'''


def main():
    """main"""


if __name__ == "__main__":
    main()



'''
quick httpx.get
'''
import sys

import argparse
# from asyncio import get_event_loop, new_event_loop, set_event_loop
from pprint import pprint
import httpx

import pytest
from loguru import logger

from .make_url import make_url
# from arun import arun

HEADERS = {
    'User-Agent':
    'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1309.0 Safari/537.17'  # noqa
}


async def httpx_get(url, proxy=None, timeout=4, verify=False, connect_timeout=8.0, headers=None, cookies=None):  # pylint: disable=too-many-arguments
    ''' quick httpx.get

    >>> arun(httpx_get('http://httpbin.org/user-agent')).json()['user-agent'][:6] in ['python', 'Mozill']  # noqa
    True
    >>> arun(httpx_get('http://httpbin.org/user-agent', headers={'User-Agent': 'xxx'})).json()['user-agent']  # noqa
    'xxx'

    loop = get_event_loop()
    if loop.is_closed():
        loop = new_event_loop()
        set_event_loop(loop)
    '''

    # logger.debug(f'args: {args}')
    # logger.debug(f'kwargs: {kwargs}')

    # for creating httpx.Response in case Exception
    req = httpx.models.Request('GET', url)

    timeo = httpx.Timeout(timeout, connect_timeout=connect_timeout)

    if headers is None:
        headers = HEADERS

    # timeout exception
    try:
        async with httpx.AsyncClient(
                proxies=proxy,
                timeout=timeo,
                trust_env=False,
                headers=headers,
                verify=verify,
                cookies=cookies,
        ) as client:
            try:
                # resp = loop.run_until_complete(client.get(*args))
                resp = await client.get(url)
                # resp.raise_for_status()
            except Exception as exc:
                logger.error(exc)
                resp = httpx.Response(
                    status_code=499,
                    request=req,
                    content=str(exc).encode(),
                )

    except Exception as exc:
        logger.error('timeout: %s' % exc)
        resp = httpx.Response(
            status_code=499,
            request=req,
            content=str(exc).encode(),
        )
    finally:
        logger.debug('resp: %s' % resp.text[:10])

    return resp

    # return loop.run_until_complete(httpx.get(*args, **kwargs))


# @pytest.mark.asyncio
async def test_noproxy_baidu():
    ''' test_noproxy_baidu '''

    # resp = arun(httpx_get('http://www.baidu.com'))
    resp = await httpx_get('http://www.baidu.com')
    assert 'baidu.com' in resp.headers.__str__()
    assert 'via' not in resp.headers


# @pytest.mark.asyncio
async def test_localhost8889_baidu():
    ''' test_localhost8889_baidu '''

    proxy = 'http://127.0.0.1:8889'
    # resp = arun(httpx_get('http://www.baidu.com', proxy=proxy))
    resp = await httpx_get('http://www.baidu.com', proxy=proxy)
    assert 'baidu.com' in resp.headers.__str__()
    assert 'via' in resp.headers


@pytest.mark.xfail
# @pytest.mark.asyncio
async def test_socks5_baidu():
    ''' test_noproxy_baidu '''

    proxy = 'socks5://127.0.0.1:1080'

    try:
        # resp = arun(httpx_get('http://www.baidu.com', proxy=proxy))
        resp = await httpx_get('http://www.baidu.com', proxy=proxy)

        assert 'baidu.com' in resp.headers.__str__()
        assert 'via' not in resp.headers
    except Exception:
        assert 0


def main():
    ''' main
    '''
    from asyncio import get_event_loop

    args = argparse.ArgumentParser()
    args.add_argument(
        dest='urls', nargs='*', help='urls (0 or more)', type=str)

    args.add_argument('-p', '--proxy', dest='proxy', help='proxy', type=str)
    args.add_argument(
        '-d', '--debug', dest='debug', action='store_true', help='debug mode')

    # merge into locals()
    _ = args.parse_args()
    # locals().update(vars(_))
    # globals().update(vars(_))

    # for elm in vars(_):
    # globals()[elm] = getattr(_, elm)
    debug, proxy, urls = _.debug, _.proxy, _.urls

    # default to INFO
    # if not _.debug:
    if not debug:
        logger.remove()
        logger.add(sys.stderr, level='INFO')

    logger.debug(f'\n\t args: {_}')

    logger.info(f'proxy: {proxy}')
    for url in urls:
        logger.info(f'url: {url}')
    for url in urls:
        coro = httpx_get(make_url(url), {'proxies': {'all': make_url(proxy)}})
        resp = get_event_loop().run_until_complete(coro)
        if resp:
            logger.debug(f'\n\t resp: {resp.text[:120]}')
            pprint(f'\n\t resp: {resp.text[:50]}')
            pprint(resp.headers)
    # return resp


if __name__ == '__main__':

    # import shlex
    # in ipython testrun
    # sys.argv = sys.argv[:1]
    # sys.argv.extend(shlex.split('www.baidu.com -d'))
    # sys.argv = sys.argv[:1]
    # sys.argv.extend(shlex.split('www.baidu.com -d -p 127.0.0.1:8889'))

    # pprint(main().text)

    main()



'''
preppend http:// if not present
'''
from typing import Optional

import re
import urllib  # pylint: disable=unused-import  # needed for typing
from urllib import parse

from loguru import logger


def make_url(  # pylint: disable=too-many-branches
        url: Optional[str] = None,
        schema: str = 'http',
) -> str:
    ''' prepent scheme (default http)
        http/https/socks if not present

    >>> make_url('http://127.0.0.1')
    'http://127.0.0.1'
    >>> make_url('htp://127.0.0.1')
    'http://127.0.0.1'
    >>> make_url('127.0.0.1')
    'http://127.0.0.1'
    >>> make_url('http://173.82.240.230:5000/json')
    'http://173.82.240.230:5000/json'
    '''

    if url is None:
        return None

    if not isinstance(url, (bytes, str)):  # pragma: no cover
        logger.warning(f'\n\t url provided: {url}, not str nor bytes, returning empty')
        return ''

    if isinstance(url, bytes):  # pragma: no cover
        try:
            url = url.decode()
        except Exception as exc:
            logger.error(exc)
            logger.info(' Something is not right...')
            raise

    # if not parsed.netloc:  # notloc not present probably wrong format
    if parse.urlparse(url).scheme and not parse.urlparse(url).netloc:  # pragma: no cover
        logger.warning(f'\n\t {url} probably malformed, trying to just extracting the ip')
        _ = re.search(r'\d+(\.\d+){3}', url)
        if _:
            url = _.group()

    if not parse.urlparse(url).scheme:
        try:
            _ = re.search(r'\d{1,4}(\.\d{1,4}){3}(:\d{1,4})?', url)
            if _:
                url = _.group()
        except Exception as exc:  # pragma: no cover
            logger.error(f' {str(exc)} ')
            logger.info('Unable to retrieve any valid ip, setting to 127.0.0.1')
            # url = '127.0.0.1'
            raise

    parsed = parse.urlparse(url)  # type: urllib.parse.ParseResult

    # proto not given, change to http,
    # if not parsed.scheme and parsed.path:

    if parsed.scheme not in ['http', 'https', 'ftp', 'socks4', 'socks5']:
        if parsed.netloc:  # pragma: no cover
            netloc = parsed.netloc
        else:
            _ = re.search(r'\d{1,4}(\.\d{1,4}){3}(:\d{1,4})?', parsed.path)
            if _:  # attempt to retrieve an ip:port pair
                path_info = _.group()
            else:  # pragma: no cover
                path_info = parsed.path

            netloc = path_info
        p_dict = {**parsed._asdict(), **{
            # 'scheme': 'http',
            'scheme': schema,
            'netloc': netloc,
            'path': '',
        }}  # type: dict

        parsed = parse.ParseResult(**p_dict)

        url = parse.urlunparse(parsed)

    return url
